
import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:indigitall_flutter_plugin/core/utils/IndigitallParams.dart';
// import 'package:flutter_webview_plugin/flutter_webview_plugin.dart';
import 'package:indigitall_flutter_plugin/core/utils/PreferenceUtils.dart';
import 'package:indigitall_flutter_plugin/inapp/models/InApp.dart';

import 'package:indigitall_flutter_plugin/push/models/Device.dart';
import 'package:indigitall_flutter_plugin/core/models/ErrorModel.dart';
import 'package:indigitall_flutter_plugin/push/models/Push.dart';
import 'package:indigitall_flutter_plugin/push/models/Topic.dart';
// import 'package:webview_flutter/webview_flutter.dart';

import 'package:indigitall_flutter_plugin/inbox/callbacks/InboxAuthListener.dart';
import 'package:indigitall_flutter_plugin/inbox/models/Inbox.dart';
import 'package:indigitall_flutter_plugin/inbox/models/InboxCounter.dart';
import 'package:indigitall_flutter_plugin/inbox/models/InboxNotification.dart';

class IndigitallFlutterPlugin {

  static const _ACTION_INIT = "initialize";
  static const _ACTION_DEVICE_ENABLE = "deviceEnable";
  static const _ACTION_DEVICE_DISABLE = "deviceDisable";
  static const _ACTION_TOPICS_LIST = "topicsList";
  static const _ACTION_TOPICS_SUBSCRIBE = "topicsSubscribe";
  static const _ACTION_TOPICS_UNSUBSCRIBE = "topicsUnsubscribe";
  static const _ACTION_EXTERNALCODE = "setExternalCode";
  static const _ACTION_SHOW_INAPP = "showInApp";
  static const _ACTION_SHOWTIME_FINISHED = "showTimeFinished";
  static const _ACTION_SHOW_MULTIPLE_INAPP = "showMultipleInApp";
  static const _ACTION_SHOW_POPUP = "showPopUp";
  static const _ACTION_SENDCUSTOMEVENT = "sendCustomEvent";
  static const _ACTION_GET_PACKAGE_NAME = "packageName";
  static const _REQUEST_CODE_ASK_PERMISSIONS = 50001;
  static const _ACTION_GET_PUSH = "getPush";
  static const _ACTION_LOGIN = "logIn";
  static const _ACTION_LOGOUT = "logOut";
  static const _ACTION_INAPP_IS_WAS_SHOWN = "inAppWasShown";
  static const _ACTION_GET_POPUP_SCALE = "popUpScale";
  static const _ACTION_TAP_INAPP = "tapInApp";
  static const _ACTION_GET_INBOX = "getInbox";
  static const _ACTION_GET_MESSAGE_COUNT = "getMessageCount";
  static const _ACTION_GET_NEXT_PAGE = "getNextPage";
  static const _ACTION_MASSIVE_EDIT = "massiveEditNotificationsWithSendingIdsList";
  static const _ACTION_GET_INFO_NOTIFICATION = "getInfoFromNotificationWithSendingId";
  static const _ACTION_MODIFY_STATUS = "modifyStatusFromNotificationWithSendingId";
  static const _ACTION_GET_AUTH_CONFIG = "getAuthConfig";

  static const _CALLBACK_ON_INDIGITALL_INITIALIZED = "callbackOnIndigitallInitialized";
  static const _CALLBACK_ON_NEW_USER_REGISTERED = "callbackOnNewUserRegistered";
  static const _CALLBACK_DEVICE_ENABLE = "callbackDeviceEnabled";
  static const _CALLBACK_DEVICE_DISABLE = "callbackDeviceDisEnabled";
  static const _CALLBACK_TOPIC_LIST = "callbackTopicList";
  static const _CALLBACK_TOPIC_SUBSCRIBE = "callbackTopicSubscribe";
  static const _CALLBACK_TOPIC_UNSUBSCRIBE = "callbackTopicUnsubscribe";
  static const _CALLBACK_EXTERNAL_CODE = "callbackExternalCode";
  static const _CALLBACK_SEND_CUSTOM_EVENT = "callbackSendCustomEvent";
  static const _CALLBACK_LOGIN = "callbackLogin";
  static const _CALLBACK_LOGOUT = "callbackLogout";
  static const _CALLBACK_GET_PUSH = "callbackgetPush";
  static const _CALLBACK_GET_INBOX = "callbackGetInbox";
  static const _CALLBACK_GET_MESSAGE_COUNT = "callbackGetMessageCount";
  static const _CALLBACK_GET_NEXT_PAGE = "callbackGetNextPage";
  static const _CALLBACK_MASSIVE_EDIT = "callbackMassiveEdit";
  static const _CALLBACK_GET_INFO_NOTIFICATION = "callbackGetInfoNotification";
  static const _CALLBACK_MODIFY_STATUS = "callbackModifyStatus";

  static const _WIDTH = "width";
  static const _HEIGHT = "height";

  static const _DID_DISMISSED = "didDismissed";
  static const _DID_CLOSED = "didClosed";
  static const _DID_CLICKED = "didClicked";

  static const MethodChannel _indigitall =
  const MethodChannel('indigitall_flutter_plugin');

  static Future<String> test() async {
    return await "texto";
  }

  static void init(Map params, Function(Device device)? onIndigitallInitialized, Function(Device device)? onNewUserRegistered, Function(ErrorModel error)? onErrorInitialized) async {
    print("[IND] init");
    params["productName"] = "flutter";
    params["productVersion"] = "0.1.0";
    params.forEach((key, value) {
      if (key == IndigitallParams.PARAM_URL_INAPP_API) PreferenceUtils.setInAppApiURL(value);
      if (key == IndigitallParams.PARAM_APP_KEY) PreferenceUtils.setAppKey(value);
    });
    await _indigitall.invokeMapMethod(_ACTION_INIT, params).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_ON_INDIGITALL_INITIALIZED) {
            Device device = new Device(valueMap);
            if (device.deviceId != null) PreferenceUtils.setDeviceId(device.deviceId!);
            // handler?.onSuccess(device);
            if (onIndigitallInitialized != null ) onIndigitallInitialized(device);
          }
          if (key == _CALLBACK_ON_NEW_USER_REGISTERED) {
            Device device = new Device(valueMap);
            if (device.deviceId != null) PreferenceUtils.setDeviceId(device.deviceId!);
            // handler?.onSuccess(device);
            if (onNewUserRegistered != null ) onNewUserRegistered(device);
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onErrorInitialized != null) onErrorInitialized(ErrorModel(error.code, error.message))
      }
    });
  }

  static void deviceEnable(Function(Device device)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_DEVICE_ENABLE).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_DEVICE_ENABLE){
            if (onSuccess != null) onSuccess(new Device(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void deviceDisable(Function(Device device)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_DEVICE_DISABLE).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_DEVICE_DISABLE){
            if (onSuccess != null) onSuccess(new Device(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void topicList(Function(List<Topic> topic)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_TOPICS_LIST).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_TOPIC_LIST) {
            if (onSuccess != null) onSuccess(Topic.createArray(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void topicSubscribe(List<String> params, Function(List<Topic> topic)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_TOPICS_SUBSCRIBE, params).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_TOPIC_SUBSCRIBE) {
            if (onSuccess != null) onSuccess(Topic.createArray(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void topicUnsubscribe(List<String> params, Function(List<Topic> topic)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_TOPICS_UNSUBSCRIBE, params).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_TOPIC_UNSUBSCRIBE) {
            if (onSuccess != null) onSuccess(Topic.createArray(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void setExternalCode(String params, Function(Device device)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_EXTERNALCODE, params).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_EXTERNAL_CODE){
            if (onSuccess != null) onSuccess(new Device(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void sendCustomEvent(Map params, Function()? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_SENDCUSTOMEVENT, params).then((value) =>
    {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_SEND_CUSTOM_EVENT) {
            if (onSuccess != null) onSuccess();
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void logIn(String externalId, Function(Device device)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_LOGIN, externalId).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_LOGIN){
            if (onSuccess != null) onSuccess(new Device(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void logOut(Function(Device device)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_LOGOUT).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_LOGOUT){
            if (onSuccess != null) onSuccess(new Device(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void getPush(Function(Push push)? onSuccess, Function(ErrorModel error)? onError) async {
    _indigitall.invokeMapMethod(_ACTION_GET_PUSH).then((value) => {
      if (value != null){
        value.forEach((key, valueMap){
          if (key == _CALLBACK_GET_PUSH) {
            if (onSuccess != null) onSuccess(new Push(
                new Map<String, dynamic>.from(json.decode(valueMap))));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void getAuthentication(BuildContext context) async {
    if (context is InboxAuthListener){
      Map? map = (context as InboxAuthListener?)?.getAuthConfig();
      await _indigitall.invokeMapMethod(_ACTION_GET_AUTH_CONFIG, map);
    }
  }

  static void getInbox(BuildContext context, Function(Inbox inbox)? onSuccess, Function(ErrorModel error)? onError) async {
    getAuthentication(context);
    await _indigitall.invokeMapMethod(_ACTION_GET_INBOX).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_GET_INBOX){
            if (onSuccess != null) onSuccess(new Inbox(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void getMessageCount(BuildContext context, Function(InboxCounter counters)? onSuccess, Function(ErrorModel error)? onError) async {
    getAuthentication(context);
    await _indigitall.invokeMapMethod(_ACTION_GET_MESSAGE_COUNT).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_GET_MESSAGE_COUNT){
            if (onSuccess != null) onSuccess(new InboxCounter(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void getNextPage(BuildContext context, Function(Inbox inbox)? onSuccess, Function(ErrorModel error)? onError) async {
    getAuthentication(context);
    await _indigitall.invokeMapMethod(_ACTION_GET_NEXT_PAGE).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_GET_NEXT_PAGE){
            if (onSuccess != null) onSuccess(new Inbox(valueMap));
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void massiveEditNotificationsWithSendingIdsList(BuildContext context, Map params, Function()? onSuccess, Function(ErrorModel error)? onError) async {
    getAuthentication(context);
    await _indigitall.invokeMapMethod(_ACTION_MASSIVE_EDIT, params).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_MASSIVE_EDIT){
            if (onSuccess != null) onSuccess();
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void getInfoFromNotificationWithSendingId(BuildContext context, Map param, Function(InboxNotification notification)? onSuccess, Function(ErrorModel error)? onError) async {
    getAuthentication(context);
    await _indigitall.invokeMapMethod(_ACTION_GET_INFO_NOTIFICATION, param).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_GET_INFO_NOTIFICATION){
            if (Platform.isAndroid){
              if (onSuccess != null) onSuccess(new InboxNotification(valueMap));
            }else{
              if (onSuccess != null) onSuccess(new InboxNotification(json.decode(valueMap.toString())));
            }

          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  static void modifyStatusFromNotificationWithSendingId(BuildContext context, Map params, Function(InboxNotification notification)? onSuccess, Function(ErrorModel error)? onError) async {
    getAuthentication(context);
    await _indigitall.invokeMapMethod(_ACTION_MODIFY_STATUS, params).then((value) => {
      if (value != null){
        value.forEach((key, valueMap) {
          if (key == _CALLBACK_MODIFY_STATUS){
            if (Platform.isAndroid){
              if (onSuccess != null) onSuccess(new InboxNotification(valueMap));
            }else{
              if (onSuccess != null) onSuccess(new InboxNotification(json.decode(valueMap.toString())));
            }
          }
        })
      }
    }).catchError((error) => {
      if (error is PlatformException){
        if (onError != null) onError(ErrorModel(error.code, error.message))
      }
    });
  }

  // static Future<WebviewScaffold>? showInApp(String inAppCode, ShowInAppHandler? handler) async {
  //   InAppCallback callback = new InAppCallback(handler);
  //   await InAppIndigitallDart.inAppGet(inAppCode, callback);
  //   return callback.webView!;
  // }
  //
  // static Future<List<WebviewScaffold>>? showMultipleInApp(List<String> inAppCodes, ShowInAppHandler? handler) async {
  //   InAppCallback callback = new InAppCallback(handler);
  //   List<WebviewScaffold> list = [];
  //   for (var i = 0; i < inAppCodes.length; i++) {
  //     await InAppIndigitallDart.inAppGet(inAppCodes[i], callback);
  //     if (callback.webView != null) list.add(callback.webView!);
  //   }
  //   return list;
  // }
  //
  // static Future<Container?>? showPopUp(String inAppCode, PopUpHandler? handler) async {
  //   InAppCallback callback = new InAppCallback(handler);
  //   await InAppIndigitallDart.inAppGet(inAppCode, callback);
  //   InApp? inApp = callback.inApp;
  //   if (inApp?.schema?.height != null && inApp?.schema?.width != null){
  //     Map map = Map();
  //     map[_WIDTH] = inApp?.schema?.width;
  //     map[_HEIGHT] = inApp?.schema?.height;
  //
  //     var aspectRadio = await _indigitall.invokeMethod(_ACTION_GET_POPUP_SCALE,map);
  //
  //     double aspect = double.parse(aspectRadio);
  //
  //     var webView = WebView(
  //       initialUrl: "https://indigitall-cdn.com/apps/5/contents/C5-65138542300df9a3.html",
  //       javascriptMode: JavascriptMode.unrestricted,
  //     );
  //
  //
  //
  //     return
  //       Container (
  //           width: inApp!.schema!.width!.toDouble()*aspect,
  //           height: inApp.schema!.height!.toDouble()*aspect,
  //           child:Stack(
  //             children: [
  //               webView,
  //               GestureDetector(
  //                 behavior: HitTestBehavior.translucent,
  //                 // onTapDown: (_) => print('gcf webview onTapDown $_'),
  //                 onTapUp: (_) async => {
  //                   print('gcf webview onTapUp $_'),
  //                   _indigitall.invokeMethod(_ACTION_TAP_INAPP, inApp.toMap())
  //                 },
  //                 child: Container(),
  //               ),
  //             ],
  //           )
  //       );
  //   }
  //   return null;
  // }


// await _indigitall.invokeMapMethod(_ACTION_SHOW_INAPP, webview).then((value) => {
//   if (value != null){
//     value.forEach((key, valueMap) {
//       if (key == _ACTION_ERROR){
//         _handlerInAppError(valueMap, handler);
//       }
//       if (key == _ACTION_SHOW_INAPP) {
//         _handlerOnLoad(valueMap, handler);
//       }
//       if (key == _ACTION_SHOWTIME_FINISHED){
//         _handlerShowTimeFinished(valueMap, handler);
//       }
//     })
//   }
// });


// static void showMultipleInApp(List<String> inAppCodeList, List<WebView> webviewList, ShowInAppHandler? handler) async {
//   await _indigitall.invokeMapMethod(_ACTION_SHOW_MULTIPLE_INAPP, [inAppCodeList, webviewList]).then((value) => {
//     if (value != null){
//       value.forEach((key, valueMap) {
//         if (key == _ACTION_ERROR){
//           _handlerInAppError(valueMap, handler);
//         }
//         if (key == _ACTION_SHOW_MULTIPLE_INAPP) {
//           _handlerOnLoad(valueMap, handler);
//         }
//         if (key == _ACTION_SHOWTIME_FINISHED){
//           _handlerShowTimeFinished(valueMap, handler);
//         }
//       })
//     }
//   });
// }
//
// static void showPopUp(Map<String, dynamic> map, PopUpHandler? handler) async {
//   await _indigitall.invokeMapMethod(_ACTION_SHOW_POPUP, map).then((value) => {
//     if (value != null){
//       value.forEach((key, valueMap) {
//         if (key == _ACTION_ERROR) {
//
//           _handlerInAppError(valueMap, handler);
//         }
//         if (key == _ACTION_SHOW_POPUP) {
//           _handlerOnLoad(valueMap, handler);
//         }
//         if (key == _ACTION_SHOWTIME_FINISHED) {
//           _handlerShowTimeFinished(valueMap, handler);
//         }
//         if (key == DID_CLICKED) {
//           handler?.didClicked();
//         }
//         if (key == DID_CLOSED) {
//           handler?.didClosed();
//         }
//         if (key == DID_DISMISSED) {
//           handler?.didDismissed();
//         }
//       })
//     }
//   });
// }
// static void _handlerOnLoad(Map<String, dynamic> value, ShowInAppHandler? handler) {
//   WebView? webView;
//   String? inAppCode;
//   value.forEach((key, value) {
//     if (key == INAPP_CODE_PARAM) {
//       inAppCode = value.toString();
//     }
//     if (key == WEBVIEW_PARAM) {
//       webView = value as WebView;
//     }
//   });
//   handler?.onLoad(inAppCode, webView);
// }
//
// static void _handlerShowTimeFinished(Map<String, dynamic> value, ShowInAppHandler? handler) {
//   int showtime = 0;
//   WebView? webView;
//   String? inAppCode;
//   value.forEach((key, value) {
//     if (key == INAPP_CODE_PARAM) {
//       inAppCode = value.toString();
//     }
//     if (key == WEBVIEW_PARAM) {
//       webView = value as WebView;
//     }
//     if (key == SHOWTIME_PARAM) {
//       showtime = int.parse(value.toString());
//     }
//   });
//   handler?.onShowTimeFinished(inAppCode, webView, showtime);
// }
//
// static void _handlerInAppError(Map<String, dynamic> errorMap, ShowInAppHandler? handler) {
//   var message = "";
//   WebView? webView;
//   String? inAppCode;
//   errorMap.forEach((key, value) {
//     if (key == INAPP_CODE_PARAM) {
//       inAppCode = value.toString();
//     }
//     if (key == WEBVIEW_PARAM) {
//       webView = value as WebView;
//     }
//     if (key == MESSAGE_PARAM) {
//       message = value.toString();
//     }
//   });
//   handler?.onInAppError(inAppCode, webView, message);
// }
}


































